package List::Moreutils::Impl::Sno;

use 5.008001;
use strict;
use warnings;

our $VERSION   = '0.400_004';

use List::MoreUtils::XS qw(); # try load XS stuff ...

=pod

=head1 NAME

List::MoreUtils::Impl::Sno - Provide after merge List::MoreUtils implementation

=head1 SYNOPSIS

  use List::MoreUtils qw(:sno);

=head1 FUNCTIONS

Currently none

=head1 SEE ALSO

L<List::Util>

=head1 AUTHOR

Jens Rehsack E<lt>rehsack AT cpan.orgE<gt>

Adam Kennedy E<lt>adamk@cpan.orgE<gt>

Tassilo von Parseval E<lt>tassilo.von.parseval@rwth-aachen.deE<gt>

=head1 COPYRIGHT AND LICENSE

Some parts copyright 2011 Aaron Crane.

Copyright 2004 - 2010 by Tassilo von Parseval
Copyright 2013 by Jens Rehsack

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut

1;
